package net.wanji.opt.service.impl;

import cn.hutool.core.util.ObjectUtil;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.enums.CrossStatusEnum;
import net.wanji.common.utils.tool.TimeArrayUtil;
import net.wanji.databus.bo.CrossIdAndStartEndDateBO;
import net.wanji.databus.dao.mapper.CrossDataHistMapper;
import net.wanji.databus.dto.MetricHistDTO;
import net.wanji.databus.po.CrossDataHistPO;
import net.wanji.opt.bo.AbnormalDetailBO;
import net.wanji.opt.service.SceneEvaluateService;
import net.wanji.opt.vo.RunningEvaluateMetricsDetailVO;
import net.wanji.opt.vo.SceneEvaluateAbnormalDetailVO;
import net.wanji.opt.vo.SceneEvaluateAbnormalDistributeVO;
import org.jetbrains.annotations.NotNull;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author Kent HAN
 * @date 2023/8/28 16:38
 */
@Slf4j
@Service
public class SceneEvaluateServiceImpl implements SceneEvaluateService {
    private final CrossDataHistMapper crossDataHistMapper;
    private final RunningEvaluateServiceImpl runningEvaluateService;
    SimpleDateFormat HOUR_SDF = new SimpleDateFormat("HH:mm");

    public SceneEvaluateServiceImpl(CrossDataHistMapper crossDataHistMapper, RunningEvaluateServiceImpl runningEvaluateService) {
        this.crossDataHistMapper = crossDataHistMapper;
        this.runningEvaluateService = runningEvaluateService;
    }

    @Override
    public SceneEvaluateAbnormalDistributeVO abnormalDistribute(CrossIdAndStartEndDateBO bo) {
        String crossId = bo.getCrossId();
        Date startDate = bo.getStartDate();
        Date endDate = bo.getEndDate();

        SceneEvaluateAbnormalDistributeVO vo = new SceneEvaluateAbnormalDistributeVO();
        List<SceneEvaluateAbnormalDistributeVO.TimeDistribution> timeDistributionList =
                buildTimeDistributionList(crossId, startDate, endDate);
        vo.setTimeDateDistributionList(buildFinalTimeDistributionList(timeDistributionList));

        setCounts(timeDistributionList, vo);
        return vo;
    }

    @Override
    public SceneEvaluateAbnormalDetailVO abnormalDetail(AbnormalDetailBO abnormalDetailBO) {
        String crossId = abnormalDetailBO.getCrossId();
        String hourMinute = abnormalDetailBO.getHourMinute();
        Date problemDate = abnormalDetailBO.getProblemDate();
        String status = abnormalDetailBO.getStatus();

        // 获取当前年份
        Calendar calendar = Calendar.getInstance();
        int currentYear = calendar.get(Calendar.YEAR);
        // 设置 problemDate 的年份为当前年份
        calendar.setTime(problemDate);
        calendar.set(Calendar.YEAR, currentYear);
        Date newDate = calendar.getTime();

        calendar.setTime(newDate);
        // 设置时间为当天开始时间（00:00:00）
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        int startOfDayTimestamp = (int) (calendar.getTimeInMillis() / 1000);
        // 设置时间为当天结束时间（23:59:59）
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        int endOfDayTimestamp = (int) (calendar.getTimeInMillis() / 1000);

        // 查询当天所有场景
        List<CrossDataHistPO> crossDataHistPOList =
                crossDataHistMapper.selectByCrossIdAndStartEnd(crossId, startOfDayTimestamp, endOfDayTimestamp);
        // 过滤场景类型


        return null;
    }

    public List<SceneEvaluateAbnormalDistributeVO.TimeDateDistribution> buildFinalTimeDistributionList(
            List<SceneEvaluateAbnormalDistributeVO.TimeDistribution> timeDistributionList) {
        List<SceneEvaluateAbnormalDistributeVO.FinalTimeDistribution> finalTimeDistributionList = new ArrayList<>();

        for (SceneEvaluateAbnormalDistributeVO.TimeDistribution timeDistribution : timeDistributionList) {
            Date problemDate = timeDistribution.getProblemDate();

            for (RunningEvaluateMetricsDetailVO.ProblemStatus problemStatus : timeDistribution.getProblemStatusList()) {
                Integer status = problemStatus.getStatus();
                Date startTime = problemStatus.getStartTime();
                Date endTime = problemStatus.getEndTime();
                // 生成 FinalTimeDistribution 对象
                for (Date time = getStartTimeOfDay(); time.before(getEndTimeOfDay()); time = addFiveMinutes(time)) {
                    if (isTimeIntersecting(time, addFiveMinutes(time), startTime, endTime)) {
                        SceneEvaluateAbnormalDistributeVO.FinalTimeDistribution finalTimeDistribution =
                                new SceneEvaluateAbnormalDistributeVO.FinalTimeDistribution();
                        finalTimeDistribution.setProblemDate(problemDate);
                        finalTimeDistribution.setStatus(status);
                        // 将 time 中的年、月、日设置为 problemDate 中的年、月、日
                        Calendar calTime = Calendar.getInstance();
                        Calendar calProblemDate = Calendar.getInstance();
                        calTime.setTime(time);
                        calProblemDate.setTime(problemDate);

                        calTime.set(Calendar.YEAR, calProblemDate.get(Calendar.YEAR));
                        calTime.set(Calendar.MONTH, calProblemDate.get(Calendar.MONTH));
                        calTime.set(Calendar.DAY_OF_MONTH, calProblemDate.get(Calendar.DAY_OF_MONTH));

                        Date newTime = calTime.getTime();
                        finalTimeDistribution.setDistributionTime(newTime);
                        finalTimeDistributionList.add(finalTimeDistribution);
                    }
                }
            }
            // 填满 finalTimeDistributionList
            // 生成时间列表
            List<Date> timeList = new ArrayList<>();
            Calendar cal = Calendar.getInstance();
            cal.setTime(problemDate);
            cal.set(Calendar.HOUR_OF_DAY, 0);
            cal.set(Calendar.MINUTE, 0);
            cal.set(Calendar.SECOND, 0);
            cal.set(Calendar.MILLISECOND, 0);

            for (int i = 0; i < 24 * 60; i += 5) {
                timeList.add(cal.getTime());
                cal.add(Calendar.MINUTE, 5);
            }

            for (Date time : timeList) {
                boolean exists = finalTimeDistributionList.stream()
                        .anyMatch(item -> item.getDistributionTime().equals(time)
                                && isSameDay(item.getProblemDate(), problemDate));
                if (!exists) {
                    SceneEvaluateAbnormalDistributeVO.FinalTimeDistribution newItem =
                            new SceneEvaluateAbnormalDistributeVO.FinalTimeDistribution();
                    newItem.setProblemDate(problemDate);
                    newItem.setDistributionTime(time);
                    newItem.setStatus(null);
                    finalTimeDistributionList.add(newItem);
                }
            }
        }
        // 重新排序
        finalTimeDistributionList
                .sort(Comparator.comparing(SceneEvaluateAbnormalDistributeVO.FinalTimeDistribution::getProblemDate)
                .thenComparing(SceneEvaluateAbnormalDistributeVO.FinalTimeDistribution::getDistributionTime));
        // 生成时间日期分布
        List<SceneEvaluateAbnormalDistributeVO.TimeDateDistribution> timeDateDistributionList = new ArrayList<>();
        List<String> minuteSectionArray = TimeArrayUtil.getMinuteSectionArray(5);
        for (String timeStr : minuteSectionArray) {
            SceneEvaluateAbnormalDistributeVO.TimeDateDistribution timeDateDistribution =
                    new SceneEvaluateAbnormalDistributeVO.TimeDateDistribution();
            timeDateDistribution.setHourMinute(timeStr);
            timeDateDistribution.setStatusByDateList(buildStatusByDateList(timeStr, finalTimeDistributionList));
            timeDateDistributionList.add(timeDateDistribution);
        }
        return timeDateDistributionList;
    }

    private List<SceneEvaluateAbnormalDistributeVO.StatusByDate> buildStatusByDateList(
            String timeStr, List<SceneEvaluateAbnormalDistributeVO.FinalTimeDistribution> finalTimeDistributionList) {
        List<SceneEvaluateAbnormalDistributeVO.StatusByDate> res = new ArrayList<>();

        for (SceneEvaluateAbnormalDistributeVO.FinalTimeDistribution ftd : finalTimeDistributionList) {
            Date problemDate = ftd.getProblemDate();
            Date distributionTime = ftd.getDistributionTime();
            String format = HOUR_SDF.format(distributionTime);
            if (Objects.equals(timeStr, format)) {
                SceneEvaluateAbnormalDistributeVO.StatusByDate inRes = getRecordInRes(res, problemDate);
                // 如果 res 中没有记录
                if (ObjectUtil.isEmpty(inRes)) {
                    SceneEvaluateAbnormalDistributeVO.StatusByDate statusByDate = buildStatusByDate(ftd, problemDate);
                    res.add(statusByDate);
                } else {
                    modifyRecordInRes(ftd, inRes);
                }
            }
        }
        return res;
    }

    private void modifyRecordInRes(
            SceneEvaluateAbnormalDistributeVO.FinalTimeDistribution ftd,
            SceneEvaluateAbnormalDistributeVO.StatusByDate inRes) {
        Integer status = ftd.getStatus();
        if (Objects.equals(status, CrossStatusEnum.UNBALANCE.getCode())) {
            inRes.setIsUnbalance(1);
        } else if (Objects.equals(status, CrossStatusEnum.CONGESTION.getCode())) {
            inRes.setIsCongestion(1);
        } else if (Objects.equals(status, CrossStatusEnum.SPILLOVER.getCode())) {
            inRes.setIsSpillover(1);
        }
    }

    @NotNull
    private SceneEvaluateAbnormalDistributeVO.StatusByDate buildStatusByDate(
            SceneEvaluateAbnormalDistributeVO.FinalTimeDistribution ftd, Date problemDate) {
        SceneEvaluateAbnormalDistributeVO.StatusByDate statusByDate =
                new SceneEvaluateAbnormalDistributeVO.StatusByDate();
        statusByDate.setProblemDate(problemDate);
        modifyRecordInRes(ftd, statusByDate);
        return statusByDate;
    }

    private SceneEvaluateAbnormalDistributeVO.StatusByDate getRecordInRes(
            List<SceneEvaluateAbnormalDistributeVO.StatusByDate> res, Date problemDate) {
        for (SceneEvaluateAbnormalDistributeVO.StatusByDate statusByDate : res) {
            Date problemDateInRes = statusByDate.getProblemDate();
            // 创建Calendar实例
            Calendar cal1 = Calendar.getInstance();
            Calendar cal2 = Calendar.getInstance();
            // 设置Calendar实例的时间
            cal1.setTime(problemDate);
            cal2.setTime(problemDateInRes);
            // 比较两个日期是否相等（忽略时间部分）
            boolean isSameDay = cal1.get(Calendar.YEAR) == cal2.get(Calendar.YEAR) &&
                    cal1.get(Calendar.DAY_OF_YEAR) == cal2.get(Calendar.DAY_OF_YEAR);
            if (isSameDay) {
                return statusByDate;
            }
        }
        return null;
    }

    private boolean isSameDay(Date date1, Date date2) {
        Calendar cal1 = Calendar.getInstance();
        Calendar cal2 = Calendar.getInstance();
        cal1.setTime(date1);
        cal2.setTime(date2);
        return cal1.get(Calendar.YEAR) == cal2.get(Calendar.YEAR) &&
                cal1.get(Calendar.DAY_OF_YEAR) == cal2.get(Calendar.DAY_OF_YEAR);
    }

    // 辅助方法：判断两个时间段是否相交
    public boolean isTimeIntersecting(Date eventStartTime, Date eventEndTime,
                                             Date sectionStartTime, Date sectionEndTime) {
        Calendar cal = java.util.Calendar.getInstance();

        cal.setTime(eventStartTime);
        int eventStartHour = cal.get(java.util.Calendar.HOUR_OF_DAY);
        int eventStartMinute = cal.get(java.util.Calendar.MINUTE);

        cal.setTime(eventEndTime);
        int eventEndHour = cal.get(java.util.Calendar.HOUR_OF_DAY);
        int eventEndMinute = cal.get(java.util.Calendar.MINUTE);

        cal.setTime(sectionStartTime);
        int sectionStartHour = cal.get(Calendar.HOUR_OF_DAY);
        int sectionStartMinute = cal.get(Calendar.MINUTE);

        cal.setTime(sectionEndTime);
        int sectionEndHour = cal.get(Calendar.HOUR_OF_DAY);
        int sectionEndMinute = cal.get(Calendar.MINUTE);

        // 将小时和分钟转换为分钟数以便比较
        int eventStartInMinutes = eventStartHour * 60 + eventStartMinute;
        int eventEndInMinutes = eventEndHour * 60 + eventEndMinute;
        int sectionStartInMinutes = sectionStartHour * 60 + sectionStartMinute;
        int sectionEndInMinutes = sectionEndHour * 60 + sectionEndMinute;

        // 判断两个时间段是否相交
        return (eventStartInMinutes < sectionEndInMinutes && eventEndInMinutes > sectionStartInMinutes) ||
                (sectionStartInMinutes < eventEndInMinutes && sectionEndInMinutes > eventStartInMinutes);
    }

    // 辅助方法：获取当天的开始时间（00:00）
    private static Date getStartTimeOfDay() {
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        return calendar.getTime();
    }

    // 辅助方法：获取当天的结束时间（23:59）
    private static Date getEndTimeOfDay() {
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 999);
        return calendar.getTime();
    }

    // 辅助方法：给时间加上5分钟
    private static Date addFiveMinutes(Date time) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(time);
        calendar.add(Calendar.MINUTE, 5);
        return calendar.getTime();
    }

    private static void setCounts(List<SceneEvaluateAbnormalDistributeVO.TimeDistribution> timeDistributionList,
                                  SceneEvaluateAbnormalDistributeVO vo) {
        int congestionDuration = 0;
        int congestionTimes = 0;
        int unbalanceDuration = 0;
        int unbalanceTimes = 0;
        int spilloverDuration = 0;
        int spilloverTimes = 0;
        for (SceneEvaluateAbnormalDistributeVO.TimeDistribution timeDistribution : timeDistributionList) {
            List<RunningEvaluateMetricsDetailVO.ProblemStatus> problemStatusList =
                    timeDistribution.getProblemStatusList();
            for (RunningEvaluateMetricsDetailVO.ProblemStatus problemStatus : problemStatusList) {
                Integer status = problemStatus.getStatus();
                Integer durationMinutes = problemStatus.getDurationMinutes();
                if (status != null && status.equals(CrossStatusEnum.CONGESTION.getCode())) {
                    congestionTimes++;
                    congestionDuration += durationMinutes;
                } else if (status != null && status.equals(CrossStatusEnum.UNBALANCE.getCode())) {
                    unbalanceTimes++;
                    unbalanceDuration += durationMinutes;
                } else if (status != null && status.equals(CrossStatusEnum.SPILLOVER.getCode())) {
                    spilloverTimes++;
                    spilloverDuration += durationMinutes;
                }
            }
        }
        vo.setCongestionTimes(congestionTimes);
        vo.setUnbalanceTimes(unbalanceTimes);
        vo.setSpilloverTimes(spilloverTimes);
        vo.setCongestionDuration(congestionDuration);
        vo.setUnbalanceDuration(unbalanceDuration);
        vo.setSpilloverDuration(spilloverDuration);
    }

    private List<SceneEvaluateAbnormalDistributeVO.TimeDistribution> buildTimeDistributionList(
            String crossId, Date startDate, Date endDate) {
        List<SceneEvaluateAbnormalDistributeVO.TimeDistribution> res = new ArrayList<>();
        Calendar start = Calendar.getInstance();
        Calendar end = Calendar.getInstance();
        start.setTime(startDate);
        end.setTime(endDate);
        // 使 endDate 包含在内
        end.add(Calendar.DATE, 1);

        for (Date date = start.getTime(); start.before(end); start.add(Calendar.DATE, 1), date = start.getTime()) {
            SceneEvaluateAbnormalDistributeVO.TimeDistribution timeDistribution =
                    new SceneEvaluateAbnormalDistributeVO.TimeDistribution();
            timeDistribution.setProblemDate(date);
            // 获取问题列表
            // 路口级别全量数据
            int startStamp = (int) (startDate.getTime() / 1000); // 10位时间戳
            int endStamp = (int) (endDate.getTime() / 1000);
            List<MetricHistDTO> crossDTOList = crossDataHistMapper.selectMetricHistDTO(
                    crossId, startStamp, endStamp);
            // 过滤有问题的记录
            List<MetricHistDTO> filteredList = crossDTOList.stream()
                    .filter(metricHistDTO -> metricHistDTO.getStatus() != null && !metricHistDTO.getStatus().equals(0))
                    .collect(Collectors.toList());
            timeDistribution.setProblemStatusList(runningEvaluateService.buildProblemStatusList(filteredList));
            res.add(timeDistribution);
        }
        return res;
    }
}
